clear 
// Uses scilab routine "ode with a stopping time" to calculate the time-evolution 
// of a double pendulum until it hits a horizontal or an inclined plane.
// This is the "main" code, which calls all the functions

	// dpendd.sci			// dynamic equations of the pendulum
	// energy_vector.sci 		// calculates KE and PE
	// animation.sci 			// does animation
	// transition.sci			// to calculate the transition matrix
	// touch.sci	 		// determine the swing leg touching ground

	// Code By: Ambarish Goswami
	// INRIA Rhone-Alpes, Grenoble, France
	// Copyright: 1994, 1995, 1996

d2r = %pi/180;	// degree to radian conversion
r2d = 180/%pi;	// radian to degree conversion
m = 5;		// leg mass
mh = 2 * m;		// hip mass
mk = mh + m;	
a = 0.5;		// a and b are portions of leg above
			// and below the leg mass location
b = 1 * a;
l = a + b;		// total leg length
g = 9.8;
t0 = 0;		// start time
t = 0: .01 : 2;	// simulation time vector
delta = 0.0001;	// time step
phi = 3*d2r;	// ground slope = 3 degrees


// each of the followings are the initial condition states
//z0 = [non-support_angle; support_angle; non-support velocity; support_velocity]
// that converge to a stable limit cycle


// The following conditions are right on the limit cycle for 
// SLOPE = 1 DEGREE
// z0 = [-11.713867*d2r;    9.713867*d2r;   -34.688622*d2r;  -49.347404*d2r]; 

// The following conditions are right on the limit cycle for 
// SLOPE = 2 DEGREE
// z0 = [-15.536893*d2r;    11.536893*d2r;  -29.732179*d2r;  -57.99031*d2r];

// The following conditions are right on the limit cycle for 
// SLOPE = 3 DEGREE
// z0 = [-18.528797*d2r; 12.528797*d2r; -21.610919*d2r; -62.557057*d2r];

// The following initial states eventually converges to limit cycle.
// SLOPE = 3 DEGREE
// z0 = [-13.6*d2r; 	13.6*d2r; 		-78*d2r; 		-71*d2r]; 	
// z0 = [-19.58040*d2r;  10.580402*d2r; 	-12.748494*d2r;   -54.102553*d2r];
// z0 = [-18.528797*d2r; 12.528797*d2r;    -21.610919*d2r;   -62.557057*d2r];

// The following initial states are for
// SLOPE = 4 DEGREE
// z0 = [- 21.12699*d2r;    13.12699*d2r;   - 12.388625*d2r;  - 65.279246*d2r];

// The following conditions are right on the limit cycle for 
// SLOPE = 5 DEGREE
// z0 = [-24.608387*d2r;     14.608387*d2r;   -1.6521924*d2r;   -68.823289*d2r];

// The following initial states eventually converges to limit cycle.
// SLOPE = 5 DEG
// z0 = [-20.105095*d2r;  9.105095*d2r;  9.791368*d2r;  -55.184301*d2r];
// z0 = [-22.105095*d2r; 10.105095*d2r;  9.891368*d2r ; -55.184301*d2r];
// z0 = [-22.479882*d2r; 12.479882*d2r; -6.1852255*d2r; -66.511967*d2r];

getf('dpendd.sci','c');			// dynamic equations of the pendulum
getf('energy_vector.sci','c') 	// calculates KE and PE
getf('animation.sci','c') 		// does animation
getf('transition.sci','c')		// to calculate the transition matrix
getf('touch.sci','c')	 		// determine the swing leg touching ground

ncc = 1;		// parameter for integrator "ode"
k = 1;		// number of steps I want the robot to walk
d = [0 1 0 0;  1 0 0 0; 0 0 0 1;  0 0 1 0]; // matrix multiplies to the state vector
							  // to switch states at transition
xbasc(0);				//deletes the contents of a graphics window
xtipt(1) = 0;			// x,y coordinates of the tip of the swing foot
ytipt(1) = 0;
kk = -1;			// index that flips after every step
				// used to keep track if it is a left or a right step
//{
for k = 1 : 4		// step number counter
	[z, rd] = ode('root', z0, t0, t, dpendd, ncc, dist); // calling the integrator "solver"
	timep(k) = rd(1);
	if k == 1 then	
		zz = z;
	elseif  kk > 0 then
		z1 = d * z;		
		zz = [zz  z1];		// append the current state to a storage state matrix
	else
		zz = [zz  z];		// append the current state to a storage state matrix
	end
//	calculation of the inertia matrix
//	dret = -m*l*b*cos(z(1,maxi(size(z)))-z(2,maxi(size(z))));
//	inert = [m*b^2  dret; dret mk*l^2+m*a^2];
	z_s = size(z);
	[ke, pe, pe1] = energy_vector(z, ytipt(k), kk);
	e = [ke pe pe1];
	if k == 1 then	
		ee = e;
	else
		ee = [ee; e];		// append the current energies to a storage energy matrix
	end

	kk = -kk;
	pp(z, xtipt(k), ytipt(k));	// append the current robot foot tip coordinates to a storage matrix

//calculation of x-displacement and y-displacement of foot in the current step
	xtipt_inc(k) = l*(sin(z(1, z_s(2))) - sin(z(2, z_s(2))));
	ytipt_inc(k) = l*(-cos(z(1, z_s(2))) + cos(z(2, z_s(2))));
	step_length(k) = sqrt(xtipt_inc(k)^2 + ytipt_inc(k)^2);

//calculation of total x-displacement and y-displacement of foot
	xtipt(k+1) = xtipt(k) + xtipt_inc(k);
	ytipt(k+1) = ytipt(k) + ytipt_inc(k);


//	z0f = [non-support_angle; support_angle; non-support_velocity; support_velocity] 
//    The termainal state for the step just completed

	z0f = [z(1, z_s(2)); z(2, z_s(2)); z(3, z_s(2)); z(4, z_s(2))];
	alpha = z0f(2) + phi;			// interleg angle
	interleg(k) = alpha*r2d;		// k-th gait step interleg angle is stored in a vector
	u = transition(alpha); 			// transition matrix
//	inv(u);
   	new_vel = u * [z0f(3,1); z0f(4,1)]; //initial velocities for next step
	if (l*cos(z(2)) < 0.5*l) then		//a condition which says don't continue
							// the simulation if the step length is too small
							// ie the foot has hit the ground too
							// prematurely.
							// There can be other, better conditions for the same
		break
	end
	z0f = [z0f(1,1); z0f(2,1); new_vel(1,1); new_vel(2,1)];
	z0 = d*z0f;					//[z0f(2); z0f(1); z0f(4); z0f(3)];
	k = k + 1;					// increase step number by one
end
//} end k

if kk < 0
	zz = r2d*[zz z0];
else
	zz = r2d*[zz d*z0];
end

zz(1,:) = modulo(zz(1,:), 360);
zz(2,:) = modulo(zz(2,:), 360);
zz_s = size(zz);

// below: keep joint angles within limits
for j = 1 : 2
	for i = 1 : zz_s(2)
		if zz(j, i) > 180 then		
			zz(j,i) = -360 + zz(j, i);
		elseif zz(j, i) < -180 then	
			zz(j,i) = 360 + zz(j, i);
		end;		
	end;		
end;

//Below: plotting code
pl1 = max(ee(:,1));
pl2 = max(ee(:,3));
pl3 = min(ee(:,3));
sh = pl2 - pl3;
pl4 = pl1 - sh;
v1 = [pl1; pl4];
v2 = [pl3; pl2];
xset("dashes",4); 
xpoly(v1, v2,"lines",0) 
plot2d(ee(:,1),ee(:,3));
